###############################################################################
#  Name:                                        Timebomb
#  Author:                                      jotham.read@gmail.com
#  Web Site:                                    http://radiosaurus.sporkism.org
#  Eggdrop Version:     1.6.x
#  Description:
#
#  This is a small TCL script for Eggdrop.  Timebomb is a game where one person
#  asks the Eggdrop bot to plant a timebomb in another users pants.  The target
#  user then needs to diffuse the bomb by cutting the correct wire, or be
#  kicked from the channel.
#
#  To start the game a user must type:
#    timebomb <nickname>
#  This will cause the target user to have a timebomb "stuffed in their pants"
#  once this occurs the user will have a number of seconds to diffuse the bomb.
#  Diffusing the bomb is done by typing:
#    cutwire <color>
#  The wire colors you can choose from are displayed when the bomb is planted.
#  This script will not allow bots (Users who are +b), or the bot running the
#  script, to be timebombed.
#
#  I know it sounds very silly but it is a rather fun game.
#
###############################################################################

bind  pub   -   !bomba  doTimebomb
bind  pub   -   !prestrihnu   doCutWire

###############################################################################
# Configuration
#

set gTimebombMinimumDuration 40
set gTimebombMaximumDuration 60
set gWireChoices "cerveny oranzovy zluty zeleny modry indigo fialovy cerny bily sedivy hnedy ruzovy bezovy"
set gMaxWireCount 3

# omezeni na kanal
set bomb_chan_on "#hrosik #kviz"

# Tyto hlasky se budou pouzivat pri detonaci
set detonatemsg [list "Bomba vybuchla a rozmetala ho do okoli!" "Bomba vybuchla a rozparala mu drzku" "Bomba vybuchla a posledni co videl, byla jeho prdel!" "Bomba vybuchla a jeho ostatky se vali uplne vsude!"]

# Ma se pri detonaci dat kick (1) nebo jen text o vybuchu (0)
set volba_detonate 1

###############################################################################
# Internal Globals
#

set gTheScriptVersion "0.4"
set gTimebombActive 0
set gTimerId 0
set gTimebombTarget ""
set gTimebombChannel ""
set gCorrectWire ""
set gNumberNames "zadny jeden dva tri ctyri pet sest sedm osm devet deset jedenact dvanact"

###############################################################################

proc note {msg} {
  putlog "% $msg"
}

proc IRCKick {theNick theChannel theReason} {
  note "Kicking $theNick in $theChannel (Reason: $theReason)"
  putserv "KICK $theChannel $theNick :$theReason"
}

proc IRCPrivMSG {theTarget messageString} {
  putserv "PRIVMSG $theTarget :$messageString"
}

proc IRCAction {theTarget messageString} {
  putserv "PRIVMSG $theTarget :\001ACTION $messageString\001"
}

proc MakeEnglishList {theList} {
  set theListLength [llength $theList]
  set returnString [lindex $theList 0]
  for {set x 1} {$x < $theListLength} {incr x} {
    if { $x == [expr $theListLength - 1] } {
      set returnString "$returnString nebo [lindex $theList $x]"
    } else {
      set returnString "$returnString, [lindex $theList $x]"
    }
  }
  return $returnString
}

proc SelectWires {wireCount} {
  global gWireChoices
  set totalWireCount [llength $gWireChoices]
  set selectedWires ""
  for {set x 0} {$x < $wireCount} {incr x} {
    set currentWire [lindex $gWireChoices [expr int( rand() * $totalWireCount )]]
    if { [lsearch $selectedWires $currentWire] == -1 } {
      lappend selectedWires $currentWire
    } else {
      set x [expr $x - 1]
    }
  }
  return $selectedWires
}

proc DetonateTimebomb {destroyTimer kickMessage} {
  global gTimebombTarget gTimerId gTimebombChannel gTimebombActive volba_detonate detonatemsg
  if { $destroyTimer } {
    killutimer $gTimerId
  }
  set gTimerId 0
  set gTimebombActive 0
  if {$volba_detonate == 1} {
   IRCKick $gTimebombTarget $gTimebombChannel $kickMessage
   } else {
  set detonatemsg_ "[lindex $detonatemsg [rand [llength $detonatemsg ]]]"
  IRCPrivMSG $gTimebombChannel "$gTimebombTarget prestrihl spatny drat. $detonatemsg_"
          }
}

proc DiffuseTimebomb {wireCut} {
  global gTimerId gTimebombActive gTimebombTarget gTimebombChannel
  killutimer $gTimerId
  set gTimerId 0
  set gTimebombActive 0
  IRCPrivMSG $gTimebombChannel "$gTimebombTarget prestrihl $wireCut drat.  Tim zneskodnil bombu!"
}

proc StartTimeBomb {theStarter theNick theChannel} {
  global gTimebombActive gTimebombTarget gTimerId gTimebombChannel gNumberNames gCorrectWire
  global gMaxWireCount gTimebombMinimumDuration gTimebombMaximumDuration
  if { $gTimebombActive == 1 } {
    note "Timebomb not started for $theStarter (Reason: timebomb already active)"
    if { $theChannel != $gTimebombChannel } {
      IRCPrivMSG $theChannel "I don't have a single bomb to spare. :-("
    } else {
      IRCAction $theChannel "pripomina, ze bombu ma $gTimebombTarget ve spodarech."
    }
  } else {
    set timerDuration [expr $gTimebombMinimumDuration + [expr int(rand() * ($gTimebombMaximumDuration - $gTimebombMinimumDuration))]]
    set gTimebombTarget $theNick
    set gTimebombChannel $theChannel
    set numberOfWires [expr 1 + int(rand() * ( $gMaxWireCount - 0 ))]
    set listOfWires [SelectWires $numberOfWires]
    set gCorrectWire [lindex $listOfWires [expr int( rand() * $numberOfWires )]]
    set wireListAsEnglish [MakeEnglishList $listOfWires]
    set wireCountAsEnglish [lindex $gNumberNames $numberOfWires]
    IRCAction $theChannel "hazi bombu... ta leti... a ma ji $gTimebombTarget ve spodarech. Display ukazuje zbyvajici cas \[\002$timerDuration\002\] vterin."
    if { $numberOfWires == 1 } {
      IRCPrivMSG $theChannel "Zneskodni bombu prestrizenim spravneho dratu. Je zde pouze $wireCountAsEnglish drat, ktery je $wireListAsEnglish. (!prestrihnu)"
    } else {
      #putserv "NOTICE $theNick :Zneskodni bombu prestrizenim spravneho dratu. Jsou zde $wireCountAsEnglish draty. Vyber $wireListAsEnglish. (!prestrihnu)"
      IRCPrivMSG $theChannel "Zneskodni bombu prestrizenim spravneho dratu. Jsou zde $wireCountAsEnglish draty. Vyber $wireListAsEnglish. (!prestrihnu)"
    }
    note "Timebomb started by $theStarter (Bomb handed to $theNick it will detonate in $timerDuration seconds)"
    set gTimebombActive 1
    set gTimerId [utimer $timerDuration "DetonateTimebomb 0 {\002*BOOM!*\002}"]
  }
}

###############################################################################
# Eggdrop command binds
#

proc doCutWire {nick uhost hand chan arg} {
  global gTimebombActive gCorrectWire gTimebombTarget bomb_chan_on detonatemsg
  if {[lsearch $bomb_chan_on [string tolower $chan]] == -1} {return}
  if { $gTimebombActive == 1 } {
    if { [string tolower $nick] == [string tolower $gTimebombTarget] } {
      if { [llength $arg] == 1 } {
        if { [string tolower $arg] == [string tolower $gCorrectWire] } {
          DiffuseTimebomb $gCorrectWire
        } else {
          DetonateTimebomb 1 "\002prestrihnuti...*BUUM!*\002"
        }
      }
    }
  }
}

proc doTimebomb {nick uhost hand chan arg} {
  global botnick bomb_chan_on theChannel
  if {[lsearch $bomb_chan_on [string tolower $chan]] == -1} {return}
  if {[onchan $arg $chan]==1 && $arg!="^RimmeR^" && $arg!="\dev\null" && $arg!="pornobaron"} {
  set theNick $nick
  if { [llength $arg] == 1 } {
    set theNick [lindex [split $arg] 0]
  }
  if { [string tolower $theNick] == [string tolower $botnick] } {
    set theNick $nick
    IRCKick $theNick $chan "Toto nebudu tolerovat!"
    return
  }
  if { [validuser $theNick] == 1 } {
    if { [matchattr $theNick "+b"] == 1 } {
      set theNick $nick
      IRCKick $theNick $chan "Nebudu to tolerovat!"
      return
    }
  }
  StartTimeBomb $nick $theNick $chan
} else {IRCPrivMSG $chan "Nech se vysetrit na ocnim, vidis tady snad nekoho s nickem $arg?!"}
}

###############################################################################

note "timebomb$gTheScriptVersion: loaded";
note " with $gMaxWireCount wire maximum,"
note " and time range of $gTimebombMinimumDuration to $gTimebombMaximumDuration seconds.";
